package com.docus.server.common.netty.server.handler;

import com.docus.core.util.DateUtil;
import com.docus.core.util.Func;
import com.docus.core.util.StringUtils;
import com.docus.core.util.json.JSON;
import com.docus.server.common.MsgConstants;
import com.docus.server.common.netty.ChannelAttribute;
import com.docus.server.common.netty.CommMsg;
import com.docus.server.common.netty.server.ChannelRepository;
import com.docus.server.common.netty.state.DeviceStateContext;
import io.netty.buffer.ByteBuf;
import io.netty.buffer.Unpooled;
import io.netty.channel.Channel;
import io.netty.channel.ChannelHandler;
import io.netty.channel.ChannelHandlerContext;
import io.netty.channel.SimpleChannelInboundHandler;
import io.netty.channel.group.ChannelGroup;
import io.netty.channel.group.DefaultChannelGroup;
import io.netty.handler.timeout.IdleStateEvent;
import io.netty.util.CharsetUtil;
import io.netty.util.concurrent.GlobalEventExecutor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.io.Serializable;
import java.net.InetSocketAddress;
import java.util.Date;

/**
 * 业务消息处理
 */
@Slf4j
@ChannelHandler.Sharable
@Component
public class NettyBusinessHandler extends SimpleChannelInboundHandler<ByteBuf> {

    private static final ChannelGroup DEFAULT_CHANNEL_GROUP = new DefaultChannelGroup(GlobalEventExecutor.INSTANCE);

    public static final ChannelAttribute<DeviceStateContext> STATE_MACHINE_SESSION = new ChannelAttribute<>("state");

    @Resource
    private ChannelRepository repository;

    @Override
    protected void channelRead0(ChannelHandlerContext ctx, ByteBuf msg) throws Exception {
        DeviceStateContext deviceStateContext = STATE_MACHINE_SESSION.getAttributeValue(ctx);

        ByteBuf buf = (ByteBuf) msg;
        //创建目标大小的数组
        byte[] barray = new byte[buf.readableBytes()];
        //把数据从bytebuf转移到byte[]
        buf.getBytes(0, barray);
        //将byte[]转成字符串用于打印
        String message = new String(barray);

        //空消息不处理
        if (!StringUtils.hasText(message)) {
            return;
        }

        CommMsg commMsg = JSON.fromJSON(message, CommMsg.class);

        String messageType = commMsg.getMessageType();
        String messageTime = commMsg.getMessageTime();
        Serializable messageContent = commMsg.getContent();

        if (messageType.equals(MsgConstants.ONLINE_REGISTER)) {
            //===========login ok,切换到已登录状态===============
            deviceStateContext.onLoginSucc("device-123", System.currentTimeMillis(), 10, "设备认证通过");

            CommMsg<Serializable> login_ok = CommMsg.builder()
                .messageTime(DateUtil.formatDateTime(new Date()))
                .content("login ok")
                .messageType("response")
                .build();
            ctx.writeAndFlush(Unpooled.copiedBuffer(JSON.toJSON(login_ok), CharsetUtil.UTF_8));

        } else if (messageType.equals(MsgConstants.HEARTBEAT_REQUEST)) {

            //============状态为上行数据=============
            deviceStateContext.onHeartbeat(System.currentTimeMillis(), "设备上行了数据");
            //==============处理数据================
            System.out.println("收到数据:" + msg);

            //============返回消息==================
            CommMsg<Serializable> resp = CommMsg.builder()
                .messageTime(DateUtil.formatDateTime(new Date()))
                .content("HEARTBEAT_REQUEST ok")
                .messageType("response HEARTBEAT_REQUEST")
                .build();
            ctx.writeAndFlush(Unpooled.copiedBuffer(JSON.toJSON(resp), CharsetUtil.UTF_8));


            log.info("接受到【终端-采集器】客户端的心跳消息：消息类型={}，消息时间={}，消息内容={}", messageType, messageTime, messageContent);

            InetSocketAddress ipSocket = (InetSocketAddress) ctx.channel().remoteAddress();
            String clientIp = ipSocket.getAddress().getHostAddress();

            log.info("【终端-采集器IP】:{}，连接上线,IP地址信息:{}", clientIp, clientIp);


        } else if (messageType.equals(MsgConstants.TERMINATOR_RESTART)) {
            log.info("接受到终端重启命令，内容{}", messageContent);

        } else if (messageType.equals(MsgConstants.COLLECTOR_RESTART)) {
            log.info("收到采集器重启命令，内容{}", messageContent);

        } else if (messageType.equals(MsgConstants.VIRTUAL_RESTART)) {
            log.info("收到虚拟机重启命令，内容{}", messageContent);

        } else if (messageType.equals(MsgConstants.UPDATE_COLLECTOR_FILE)) {
            log.info("收到更新采集器文件命令，内容{}", messageContent);

        } else if (messageType.equals(MsgConstants.UPDATE_COLLECTOR_CONFIG)) {
            log.info("收到更新采集器配置命令，内容{}", messageContent);

        } else if (messageType.equals(MsgConstants.SCH_DISTRIBUTE_TASKS)) {
            log.info("收到采集调度器下发任务命令，内容{}", messageContent);

        }


        System.out.println("channelRead0:" + deviceStateContext.toString());
    }

    /**
     * 空闲一段时间，就进行检查 (当前时间-上次上行数据的时间) 如果大于设定的超时时间 设备状态就就行一次 onTimeout
     */
    @Override
    public void userEventTriggered(ChannelHandlerContext ctx, Object evt) throws Exception {
        System.out.println(getClass().getSimpleName() + "." + "userEventTriggered" + ctx.channel().remoteAddress());
        if (evt instanceof IdleStateEvent) {
            DeviceStateContext deviceStateContext = STATE_MACHINE_SESSION.getAttributeValue(ctx);
            long lastUpdateTime = deviceStateContext.getLastUpdateTime();
            long currentTimeMillis = System.currentTimeMillis();
            long intervalTime = currentTimeMillis - lastUpdateTime;

            if (intervalTime > deviceStateContext.getHeartRate()) {
                //==============发生超时，进入超时状态==============
                deviceStateContext.onTimeout("设备发生超时");
                System.out.println("userEventTriggered:" + deviceStateContext.toString());
            }
        } else {
            //不是超时事件，进行传递
            super.userEventTriggered(ctx, evt);
        }
    }

    /**
     * netty client 上线
     */
    @Override
    public void channelActive(ChannelHandlerContext ctx) throws Exception {
        ctx.fireChannelRegistered();

        //链接成功
        DeviceStateContext deviceStateContext = new DeviceStateContext(ctx.channel(), true);
        //===========设置设备状态为 未登录=================
        deviceStateContext.onConnect(System.currentTimeMillis(), "设备 active");
        //更新添加 state 属性
        STATE_MACHINE_SESSION.setAttribute(ctx, deviceStateContext);
        System.out.println("channelActive:" + deviceStateContext.toString());

        //===========设置设备状态为 未登录=================
        InetSocketAddress ipSocket = (InetSocketAddress) ctx.channel().remoteAddress();
        String clientIp = ipSocket.getAddress().getHostAddress();
        log.info("【终端-采集器IP】:{}，连接上线,IP地址信息:{}", clientIp, clientIp);

        if (Func.isNotBlank(clientIp)) {

            repository.put(clientIp, ctx.channel());
        }

        DEFAULT_CHANNEL_GROUP.add(ctx.channel());

        System.out.println(ctx.channel().remoteAddress() + " 上线," + "【采集器-终端】在线数量：" + DEFAULT_CHANNEL_GROUP.size());
    }

    /**
     * netty client 下线
     */
    @Override
    public void channelInactive(ChannelHandlerContext ctx) throws Exception {
        Channel channel = ctx.channel();
        System.out.println(channel.remoteAddress() + " 下线," + "【采集器-终端】在线数量：" + DEFAULT_CHANNEL_GROUP.size());
        String clientId = repository.getClientKey(channel);
        log.error("客户端下线，终端连接：{}", clientId);
        //移除终端，终端离线
        if (clientId != null) {
            repository.remove(clientId);
        }

        //================设置为断开================
        DeviceStateContext deviceStateContext = STATE_MACHINE_SESSION.getAttributeValue(ctx);
        deviceStateContext.onDisconnect("设备 inactive");
        System.out.println("channelInactive:" + deviceStateContext.toString());
    }

    /**
     * netty exception 通道异常
     */
    @Override
    public void exceptionCaught(ChannelHandlerContext ctx, Throwable cause) throws Exception {
        //==============发生异常切换到断开模式===============
        System.out.println("exceptionCaught:" + cause.getMessage());
        DeviceStateContext deviceStateContext = STATE_MACHINE_SESSION.getAttributeValue(ctx);
        deviceStateContext.onDisconnect(cause.getMessage());
        System.out.println("exceptionCaught:" + deviceStateContext.toString());


        cause.printStackTrace();
        String clientId = repository.getClientKey(ctx.channel());
        log.error("通道发生异常，终端连接：{}", clientId);
        //移除终端，终端离线
        if (clientId != null) {
            repository.remove(clientId);
        }
        if (ctx.channel().isActive()) {
            ctx.close();
        }
        super.exceptionCaught(ctx, cause);
    }

}
